@@ -19,8 +19,9 @@
 package org.apache.felix.framework;
 
 import java.io.IOException;
-import java.net.InetAddress; 
+import java.net.InetAddress;
 import java.io.InputStream;
+import java.net.JarURLConnection;
 import java.net.URL;
 import java.net.URLConnection;
 import java.net.URLStreamHandler;
@@ -33,12 +34,15 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.NoSuchElementException;
+import java.util.Properties;
 import java.util.Set;
 
 import org.apache.felix.framework.util.FelixConstants;
+import org.apache.felix.framework.util.SecurityManagerEx;
 import org.apache.felix.framework.util.Util;
 import org.apache.felix.framework.util.manifestparser.Capability;
 import org.apache.felix.framework.util.manifestparser.ManifestParser;
+import org.apache.felix.framework.util.manifestparser.R4Attribute;
 import org.apache.felix.framework.util.manifestparser.R4Directive;
 import org.apache.felix.framework.util.manifestparser.R4Library;
 import org.apache.felix.moduleloader.ICapability;
@@ -56,57 +60,57 @@ import org.osgi.framework.BundleException;
 import org.osgi.framework.Constants;
 
 /**
- * The ExtensionManager class is used in several ways. 
+ * The ExtensionManager class is used in several ways.
  * <p>
- * First, a private instance is added (as URL with the instance as 
- * URLStreamHandler) to the classloader that loaded the class. 
- * It is assumed that this is an instance of URLClassloader (if not extension 
- * bundles will not work). Subsequently, extension bundles can be managed by 
- * instances of this class (their will be one instance per framework instance). 
+ * First, a private instance is added (as URL with the instance as
+ * URLStreamHandler) to the classloader that loaded the class.
+ * It is assumed that this is an instance of URLClassloader (if not extension
+ * bundles will not work). Subsequently, extension bundles can be managed by
+ * instances of this class (their will be one instance per framework instance).
  * </p>
  * <p>
  * Second, it is used as module definition of the systembundle. Added extension
- * bundles with exported packages will contribute their exports to the 
+ * bundles with exported packages will contribute their exports to the
  * systembundle export.
  * </p>
  * <p>
- * Third, it is used as content loader of the systembundle. Added extension 
+ * Third, it is used as content loader of the systembundle. Added extension
  * bundles exports will be available via this loader.
  * </p>
  */
-// The general approach is to have one private static instance that we register 
+// The general approach is to have one private static instance that we register
 // with the parent classloader and one instance per framework instance that
 // keeps track of extension bundles and systembundle exports for that framework
 // instance.
 class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IContentLoader, IContent
 {
     // The private instance that is added to Felix.class.getClassLoader() -
-    // will be null if extension bundles are not supported (i.e., we are not 
+    // will be null if extension bundles are not supported (i.e., we are not
     // loaded by an instance of URLClassLoader)
     static final ExtensionManager m_extensionManager;
-    
+
     static
     {
-        // We use the secure action of Felix to add a new instance to the parent 
-        // classloader. 
+        // We use the secure action of Felix to add a new instance to the parent
+        // classloader.
         ExtensionManager extensionManager = new ExtensionManager();
         try
         {
             Felix.m_secureAction.addURLToURLClassLoader(Felix.m_secureAction.createURL(
-                Felix.m_secureAction.createURL(null, "felix:", extensionManager),
-                "felix://extensions/", extensionManager),
+                Felix.m_secureAction.createURL(null, "http:", extensionManager),
+                "http://felix.extensions:9/", extensionManager),
                 Felix.class.getClassLoader());
         }
-        catch (Exception ex) 
+        catch (Exception ex)
         {
-            // extension bundles will not be supported. 
+            // extension bundles will not be supported.
             extensionManager = null;
         }
         m_extensionManager = extensionManager;
     }
 
     private Logger m_logger = null;
-    private BundleInfo m_systemBundleInfo = null;
+    private BundleInfo m_sbi = null;
     private ICapability[] m_capabilities = null;
     private Set m_exportNames = null;
     private ISearchPolicy m_searchPolicy = null;
@@ -127,26 +131,28 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
 
     /**
      * This constructor is used to create one instance per framework instance.
-     * The general approach is to have one private static instance that we register 
+     * The general approach is to have one private static instance that we register
      * with the parent classloader and one instance per framework instance that
      * keeps track of extension bundles and systembundle exports for that framework
      * instance.
-     * 
+     *
      * @param logger the logger to use.
      * @param config the configuration to read properties from.
      * @param systemBundleInfo the info to change if we need to add exports.
      */
-    ExtensionManager(Logger logger, Map configMap, BundleInfo systemBundleInfo)
+    ExtensionManager(Logger logger, Map configMap, BundleInfo sbi)
     {
         m_extensions = null;
         m_names = null;
         m_sourceToExtensions = null;
         m_logger = logger;
-        m_systemBundleInfo = systemBundleInfo;
+        m_sbi = sbi;
 
 // TODO: FRAMEWORK - Not all of this stuff really belongs here, probably only exports.
         // Populate system bundle header map.
-        Map map = ((SystemBundleArchive) m_systemBundleInfo.getArchive()).getManifestHeader(0);
+        // Note: This is a reference to the actual header map,
+        // so these changes are saved. Kind of hacky.
+        Map map = m_sbi.getCurrentHeader();
         // Initialize header map as a case insensitive map.
         map.put(FelixConstants.BUNDLE_VERSION,
             configMap.get(FelixConstants.FELIX_VERSION_PROPERTY));
@@ -163,12 +169,19 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         // The system bundle exports framework packages as well as
         // arbitrary user-defined packages from the system class path.
         // We must construct the system bundle's export metadata.
-        // Get system property that specifies which class path
+        // Get configuration property that specifies which class path
         // packages should be exported by the system bundle.
+        String syspkgs = (String) configMap.get(Constants.FRAMEWORK_SYSTEMPACKAGES);
+        // If no system packages were specified, load our default value.
+        syspkgs = (syspkgs == null) ? loadDefaultSystemPackages(m_logger) : syspkgs;
+        // If any extra packages are specified, then append them.
+        String extra = (String) configMap.get(Constants.FRAMEWORK_SYSTEMPACKAGES_EXTRA);
+        syspkgs = (extra == null) ? syspkgs : syspkgs + "," + extra;
         try
         {
-            setCapabilities(ManifestParser.parseExportHeader(
-                (String) configMap.get(Constants.FRAMEWORK_SYSTEMPACKAGES)));
+            setCapabilities(
+                addModuleCapability(map,
+                    ManifestParser.parseExportHeader(syspkgs)));
         }
         catch (Exception ex)
         {
@@ -176,7 +189,46 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
             m_logger.log(
                 Logger.LOG_ERROR,
                 "Error parsing system bundle export statement: "
-                + configMap.get(Constants.FRAMEWORK_SYSTEMPACKAGES), ex);
+                + syspkgs, ex);
+        }
+    }
+
+    private ICapability[] addModuleCapability(Map headerMap, ICapability[] caps)
+    {
+        try
+        {
+            // First parse the symbolic name header.
+            ICapability moduleCap = ManifestParser.parseBundleSymbolicName(headerMap);
+            // Create a copy of the module capability with the standard
+            // alias for the bundle symbolic name.
+            R4Directive[] dirs = ((Capability) moduleCap).getDirectives();
+            R4Attribute[] attrs = ((Capability) moduleCap).getAttributes();
+            R4Attribute[] attrsAlias = new R4Attribute[attrs.length];
+            System.arraycopy(attrs, 0, attrsAlias, 0, attrs.length);
+            // Modify the alias attributes to have the standard symbolic name.
+            for (int i = 0; i < attrsAlias.length; i++)
+            {
+                if (attrsAlias[i].getName().equalsIgnoreCase(Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE))
+                {
+                    attrsAlias[i] = new R4Attribute(
+                        Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE, Constants.SYSTEM_BUNDLE_SYMBOLICNAME, false);
+                }
+            }
+            // Create the alias capability.
+            ICapability aliasCap = new Capability(ICapability.MODULE_NAMESPACE, dirs, attrsAlias);
+            // Finally, add the module and alias capabilities to the
+            // existing capabilities.
+            ICapability[] temp = new ICapability[caps.length + 2];
+            System.arraycopy(caps, 0, temp, 2, caps.length);
+            temp[0] = moduleCap;
+            temp[1] = aliasCap;
+            return temp;
+        }
+        catch (BundleException ex)
+        {
+            // This should not happen, but in case it does, then just
+            // return the original array of capabilities.
+            return caps;
         }
     }
 
@@ -195,29 +247,25 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
      */
     boolean isExtensionBundle(Map headers)
     {
-        R4Directive dir = ManifestParser.parseExtensionBundleHeader((String)
-            headers.get(Constants.FRAGMENT_HOST));
-
-        return (dir != null) && (Constants.EXTENSION_FRAMEWORK.equals(
-            dir.getValue()) || Constants.EXTENSION_BOOTCLASSPATH.equals(
-            dir.getValue()));
+        return (ManifestParser.parseExtensionBundleHeader((String)
+                headers.get(Constants.FRAGMENT_HOST)) != null);
     }
 
     /**
      * Add an extension bundle. The bundle will be added to the parent classloader
-     * and it's exported packages will be added to the module definition 
-     * exports of this instance. Subsequently, they are available form the 
+     * and it's exported packages will be added to the module definition
+     * exports of this instance. Subsequently, they are available form the
      * instance in it's role as content loader.
-     * 
+     *
      * @param felix the framework instance the given extension bundle comes from.
      * @param bundle the extension bundle to add.
      * @throws BundleException if extension bundles are not supported or this is
      *          not a framework extension.
-     * @throws SecurityException if the caller does not have the needed 
+     * @throws SecurityException if the caller does not have the needed
      *          AdminPermission.EXTENSIONLIFECYCLE and security is enabled.
      * @throws Exception in case something goes wrong.
      */
-    void addExtensionBundle(Felix felix, FelixBundle bundle) 
+    void addExtensionBundle(Felix felix, FelixBundle bundle)
         throws SecurityException, BundleException, Exception
     {
         Object sm = System.getSecurityManager();
@@ -250,11 +298,7 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         {
             bundle.getInfo().setExtension(true);
 
-            SystemBundleArchive systemArchive =
-                (SystemBundleArchive) felix.getInfo().getArchive();
-
             // Merge the exported packages with the exported packages of the systembundle.
-            Map headers = null;
             ICapability[] exports = null;
             try
             {
@@ -271,7 +315,7 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
             }
 
             // Add the bundle as extension if we support extensions
-            if (m_extensionManager != null) 
+            if (m_extensionManager != null)
             {
                 // This needs to be the private instance.
                 m_extensionManager.addExtension(felix, bundle);
@@ -298,14 +342,14 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         {
             felix.releaseBundleLock(felix);
         }
-    
+
         bundle.getInfo().setState(Bundle.RESOLVED);
     }
 
     /**
      * This is a Felix specific extension mechanism that allows extension bundles
      * to have activators and be started via this method.
-     * 
+     *
      * @param felix the framework instance the extension bundle is installed in.
      * @param bundle the extension bundle to start if it has a Felix specific activator.
      */
@@ -314,7 +358,7 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         String activatorClass = (String)
         bundle.getInfo().getCurrentHeader().get(
             FelixConstants.FELIX_EXTENSION_ACTIVATOR);
-        
+
         if (activatorClass != null)
         {
             try
@@ -322,13 +366,14 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
                 BundleActivator activator = (BundleActivator)
                     felix.getClass().getClassLoader().loadClass(
                         activatorClass.trim()).newInstance();
-                
+
+// TODO: KARL - This is kind of hacky, can we improve it?
                 felix.m_activatorList.add(activator);
-                
-                BundleContext context = m_systemBundleInfo.getBundleContext();
-                
+
+                BundleContext context = m_sbi.getBundleContext();
+
                 bundle.getInfo().setBundleContext(context);
-                
+
                 if (felix.getInfo().getState() == Bundle.ACTIVE)
                 {
                     Felix.m_secureAction.startActivator(activator, context);
@@ -342,15 +387,15 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
             }
         }
     }
-    
+
     /**
      * Remove all extension registered by the given framework instance. Note, it
      * is not possible to unregister allready loaded classes form those extensions.
      * That is why the spec requires a JVM restart.
-     * 
-     * @param felix the framework instance whose extensions need to be unregistered. 
+     *
+     * @param felix the framework instance whose extensions need to be unregistered.
      */
-    void removeExtensions(Felix felix) 
+    void removeExtensions(Felix felix)
     {
         if (m_extensionManager != null)
         {
@@ -370,9 +415,10 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
     {
         m_capabilities = capabilities;
 
-        Map map = ((SystemBundleArchive) m_systemBundleInfo.getArchive()).getManifestHeader(0);
+        // Note: This is a reference to the actual header map,
+        // so these changes are saved. Kind of hacky.
+        Map map = m_sbi.getCurrentHeader();
         map.put(Constants.EXPORT_PACKAGE, convertCapabilitiesToHeaders(map));
-        ((SystemBundleArchive) m_systemBundleInfo.getArchive()).setManifestHeader(map);
     }
 
     public IRequirement[] getDynamicRequirements()
@@ -397,28 +443,34 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
 
         for (int i = 0; (m_capabilities != null) && (i < m_capabilities.length); i++)
         {
-            if (i > 0)
+            if (m_capabilities[i].getNamespace().equals(ICapability.PACKAGE_NAMESPACE))
             {
-                exportSB.append(", ");
+                // Add a comma separate if there is an existing package.
+                if (exportSB.length() > 0)
+                {
+                    exportSB.append(", ");
+                }
+
+                // Append exported package information.
+                exportSB.append(((Capability) m_capabilities[i]).getPackageName());
+                exportSB.append("; version=\"");
+                exportSB.append(((Capability) m_capabilities[i]).getPackageVersion().toString());
+                exportSB.append("\"");
+
+                // Remember exported packages.
+                exportNames.add(((Capability) m_capabilities[i]).getPackageName());
             }
-
-            exportSB.append(((Capability) m_capabilities[i]).getPackageName());
-            exportSB.append("; version=\"");
-            exportSB.append(((Capability) m_capabilities[i]).getPackageVersion().toString());
-            exportSB.append("\"");
-
-            exportNames.add(((Capability) m_capabilities[i]).getPackageName());
         }
 
         m_exportNames = exportNames;
 
         return exportSB.toString();
     }
-    
+
     //
     // IContentLoader
     //
-    
+
     public void open()
     {
         // Nothing needed here.
@@ -513,7 +565,7 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         // No native libs associated with the system bundle.
         return null;
     }
-    
+
     //
     // Classpath Extension
     //
@@ -544,10 +596,10 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         throw new IOException("Resource not provided by any extension!");
     }
 
-    protected InetAddress getHostAddress(URL u) 
-    { 
-        // the extension URLs do not address real hosts 
-        return null; 
+    protected InetAddress getHostAddress(URL u)
+    {
+        // the extension URLs do not address real hosts
+        return null;
     }
 
     private synchronized void addExtension(Object source, Bundle extension)
@@ -595,16 +647,16 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         }
     }
 
-    public Enumeration getEntries() 
+    public Enumeration getEntries()
     {
         return new Enumeration()
         {
-            public boolean hasMoreElements() 
+            public boolean hasMoreElements()
             {
                 return false;
             }
 
-            public Object nextElement() throws NoSuchElementException 
+            public Object nextElement() throws NoSuchElementException
             {
                 throw new NoSuchElementException();
             }
@@ -615,12 +667,12 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
         return false;
     }
 
-    public byte[] getEntryAsBytes(String name) 
+    public byte[] getEntryAsBytes(String name)
     {
         return null;
     }
 
-    public InputStream getEntryAsStream(String name) throws IOException 
+    public InputStream getEntryAsStream(String name) throws IOException
     {
         return null;
     }
@@ -634,4 +686,55 @@ class ExtensionManager extends URLStreamHandler implements IModuleDefinition, IC
     {
         return null;
     }
+
+    //
+    // Utility methods.
+    //
+
+    /**
+     * The default name used for the default configuration properties file.
+    **/
+    public static final String DEFAULT_PROPERTIES_FILE_VALUE = "default.properties";
+
+    static String loadDefaultSystemPackages(Logger logger)
+    {
+        // If we cannot get configuration properties for any reason, then just
+        // attempt to load resource default.properties instead.
+        URL propURL = ExtensionManager.class.getClassLoader()
+            .getResource(DEFAULT_PROPERTIES_FILE_VALUE);
+        InputStream is = null;
+        try
+        {
+            // Load properties from URL.
+            is = propURL.openConnection().getInputStream();
+            Properties props = new Properties();
+            props.load(is);
+            is.close();
+            // Perform variable substitution for system properties.
+            for (Enumeration e = props.propertyNames(); e.hasMoreElements(); )
+            {
+                String name = (String) e.nextElement();
+                props.setProperty(name,
+                    Util.substVars(props.getProperty(name), name, null, props));
+            }
+            // Return system packages property.
+            return props.getProperty(Constants.FRAMEWORK_SYSTEMPACKAGES);
+        }
+        catch (Exception ex)
+        {
+            // Try to close input stream if we have one.
+            try
+            {
+                if (is != null) is.close();
+            }
+            catch (IOException ex2)
+            {
+                // Nothing we can do.
+            }
+
+            logger.log(
+                Logger.LOG_ERROR, "Unable to load any configuration properties.", ex);
+            return "";
+        }
+    }
 }
\ No newline at end of file
