@@ -46,8 +46,6 @@ public interface FelixConstants extends org.osgi.framework.Constants
     public static final String LOG_LOGGER_PROP = "felix.log.logger";
     public static final String SYSTEMBUNDLE_ACTIVATORS_PROP
         = "felix.systembundle.activators";
-    public static final String FRAMEWORK_STARTLEVEL_PROP
-        = "org.osgi.framework.startlevel";
     public static final String BUNDLE_STARTLEVEL_PROP
         = "felix.startlevel.bundle";
     public static final String SERVICE_URLHANDLERS_PROP = "felix.service.urlhandlers";
@@ -66,4 +64,4 @@ public interface FelixConstants extends org.osgi.framework.Constants
     // Miscellaneous properties values.
     public static final String FAKE_URL_PROTOCOL_VALUE = "location:";
     public static final String FELIX_EXTENSION_ACTIVATOR = "Felix-Activator";
-}
\ No newline at end of file
+}
