@@ -21,23 +21,23 @@ package org.apache.felix.framework.util.manifestparser;
 import java.util.*;
 
 import org.apache.felix.framework.Logger;
-import org.apache.felix.framework.cache.BundleRevision;
-import org.apache.felix.framework.util.*;
+import org.apache.felix.framework.util.FelixConstants;
+import org.apache.felix.framework.util.VersionRange;
 import org.apache.felix.moduleloader.ICapability;
 import org.apache.felix.moduleloader.IRequirement;
 import org.osgi.framework.*;
 
 public class ManifestParser
 {
-    private Logger m_logger = null;
-    private Map m_configMap = null;
-    private Map m_headerMap = null;
-    private String m_bundleSymbolicName = null;
-    private Version m_bundleVersion = null;
-    private ICapability[] m_capabilities = null;
-    private IRequirement[] m_requirements = null;
-    private IRequirement[] m_dynamicRequirements = null;
-    private R4LibraryClause[] m_libraryHeaders = null;
+    private Logger m_logger;
+    private Map m_configMap;
+    private Map m_headerMap;
+    private String m_bundleSymbolicName;
+    private Version m_bundleVersion;
+    private ICapability[] m_capabilities;
+    private IRequirement[] m_requirements;
+    private IRequirement[] m_dynamicRequirements;
+    private R4LibraryClause[] m_libraryHeaders;
     private boolean m_libraryHeadersOptional = false;
 
     public ManifestParser(Logger logger, Map configMap, Map headerMap)
@@ -49,6 +49,7 @@ public class ManifestParser
 
         // Verify that only manifest version 2 is specified.
         String manifestVersion = (String) m_headerMap.get(Constants.BUNDLE_MANIFESTVERSION);
+        manifestVersion = (manifestVersion == null) ? null : manifestVersion.trim();
         if ((manifestVersion != null) && !manifestVersion.equals("2"))
         {
             throw new BundleException(
@@ -62,53 +63,53 @@ public class ManifestParser
         Map dupeMap = new HashMap();
 
         //
-        // Get bundle version.
+        // Parse bundle symbolic name.
         //
 
-        if (m_headerMap.get(Constants.BUNDLE_VERSION) != null)
+        ICapability moduleCap = parseBundleSymbolicName(m_headerMap);
+        if (moduleCap != null)
         {
-            try
+            m_bundleVersion = (Version)
+                moduleCap.getProperties().get(Constants.BUNDLE_VERSION_ATTRIBUTE);
+            m_bundleSymbolicName = (String)
+                moduleCap.getProperties().get(Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE);
+
+            // Add the module capability to all capabilities.
+            // TODO: FRAGMENT - Fragment bundles cannot be required, so we
+            //       should not add this capability, but for now we are using
+            //       it to get the symbolic name.
+            capList.add(moduleCap);
+            // Add a host capability if the bundle is not a fragment. A host
+            // capability is the same as a module capability, but with a
+            // different capability namespace.
+            if (headerMap.get(Constants.FRAGMENT_HOST) == null)
             {
-                m_bundleVersion = Version.parseVersion((String) m_headerMap.get(Constants.BUNDLE_VERSION));
-            }
-            catch (RuntimeException ex)
-            {
-                // R4 bundle versions must parse, R3 bundle version may not.
-                if (getManifestVersion().equals("2"))
-                {
-                    throw ex;
-                }
-                m_bundleVersion = Version.emptyVersion;
+                capList.add(new Capability(
+                    ICapability.HOST_NAMESPACE, null,
+                    ((Capability) moduleCap).getAttributes()));
             }
         }
 
         //
-        // Parse bundle symbolic name.
+        // Parse Fragment-Host.
         //
-
         Object[][][] clauses = parseStandardHeader(
-            (String) headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+            (String) headerMap.get(Constants.FRAGMENT_HOST));
         if (clauses.length > 0)
         {
-            if (clauses.length > 1)
+            validateFragment(headerMap);
+            try
             {
-                throw new BundleException(
-                    "Cannot have multiple symbolic names: "
-                        + headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+                reqList.add(
+                    new Requirement(
+                        ICapability.HOST_NAMESPACE,
+                        "(" + Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE
+                            + "=" + clauses[0][CLAUSE_PATHS_INDEX][0] + ")"));
             }
-            else if (clauses[0][CLAUSE_PATHS_INDEX].length > 1)
+            catch (InvalidSyntaxException ex)
             {
-                throw new BundleException(
-                    "Cannot have multiple symbolic names: "
-                        + headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+                ex.printStackTrace();
             }
-            m_bundleSymbolicName = (String) clauses[0][CLAUSE_PATHS_INDEX][0];
-            R4Attribute[] attrs = new R4Attribute[2];
-            attrs[0] = new R4Attribute(
-                Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE, m_bundleSymbolicName, false);
-            attrs[1] = new R4Attribute(
-                Constants.BUNDLE_VERSION_ATTRIBUTE, m_bundleVersion, false);
-            capList.add(new Capability(ICapability.MODULE_NAMESPACE, null, attrs));
         }
 
         //
@@ -239,10 +240,44 @@ public class ManifestParser
         }
     }
 
+    /**
+     * Checks whether a fragment uses features that we do not currently support
+     * (e.g., Import-Package, Export-Package, and Bundle-NativeCode). If so, we
+     * throw an exception.
+     * @param headerMap the header to validate.
+    **/
+    private void validateFragment(Map headerMap) throws BundleException
+    {
+        // TODO: FRAGMENTS - We should delete this method and the related
+        //       fragment constants in FelixConstants when fragments are
+        //       fully implemented.
+        String fragmentHost = (String) headerMap.get(Constants.FRAGMENT_HOST);
+        if ((fragmentHost != null) && (parseExtensionBundleHeader(fragmentHost) == null))
+        {
+            if ((headerMap.get(Constants.IMPORT_PACKAGE) != null)
+                || (headerMap.get(Constants.EXPORT_PACKAGE) != null)
+                || (headerMap.get(Constants.BUNDLE_NATIVECODE) != null))
+            {
+                String s = (String) m_configMap.get(FelixConstants.FRAGMENT_VALIDATION_PROP);
+                s = (s == null) ? FelixConstants.FRAGMENT_VALIDATION_EXCEPTION_VALUE : s;
+                if (s.equalsIgnoreCase(FelixConstants.FRAGMENT_VALIDATION_WARNING_VALUE))
+                {
+                    m_logger.log(Logger.LOG_WARNING,
+                        "Fragments with exports, imports, or native code are not currently supported.");
+                }
+                else
+                {
+                    throw new BundleException(
+                        "Fragments with exports, imports, or native code are not currently supported.");
+                }
+            }
+        }
+    }
+
     public String getManifestVersion()
     {
         String manifestVersion = (String) m_headerMap.get(Constants.BUNDLE_MANIFESTVERSION);
-        return (manifestVersion == null) ? "1" : manifestVersion;
+        return (manifestVersion == null) ? "1" : manifestVersion.trim();
     }
 
     public String getSymbolicName()
@@ -313,7 +348,7 @@ public class ManifestParser
                             clause.getLibraryEntries()[i],
                             clause.getOSNames(), clause.getProcessors(), clause.getOSVersions(),
                             clause.getLanguages(), clause.getSelectionFilter());
-                    } 
+                    }
                 }
                 if (current < libraries.length)
                 {
@@ -594,7 +629,6 @@ public class ManifestParser
                     // and add version if missing.
                     m_requirements[reqIdx] = new Requirement(
                         ICapability.PACKAGE_NAMESPACE,
-                        (String) pkgName.getValue(),
                         null,
                         new R4Attribute[] { pkgName, pkgVersion });
                 }
@@ -636,10 +670,7 @@ public class ManifestParser
 
                 map.put(
                     m_capabilities[i].getProperties().get(ICapability.PACKAGE_PROPERTY),
-                    new Requirement(
-                        ICapability.PACKAGE_NAMESPACE,
-                        (String) m_capabilities[i].getProperties().get(ICapability.PACKAGE_PROPERTY),
-                        null, attrs));
+                    new Requirement(ICapability.PACKAGE_NAMESPACE, null, attrs));
             }
         }
         m_requirements =
@@ -731,14 +762,22 @@ public class ManifestParser
                     newAttrs);
             }
         }
+
+        R4Directive extension = parseExtensionBundleHeader((String)
+            m_headerMap.get(Constants.FRAGMENT_HOST));
         
-        if (parseExtensionBundleHeader((String) 
-            m_headerMap.get(Constants.FRAGMENT_HOST)) != null)
+        if (extension != null)
         {
+            if (!(Constants.EXTENSION_FRAMEWORK.equals(extension.getValue()) || 
+                Constants.EXTENSION_BOOTCLASSPATH.equals(extension.getValue())))
+            {
+                throw new BundleException(
+                    "Extension bundle must have either 'extension:=framework' or 'extension:=bootclasspath'");
+            }
             checkExtensionBundle();
         }
     }
-    
+
     private void checkExtensionBundle() throws BundleException
     {
         if (m_headerMap.containsKey(Constants.IMPORT_PACKAGE) ||
@@ -751,6 +790,58 @@ public class ManifestParser
         }
     }
 
+    public static ICapability parseBundleSymbolicName(Map headerMap)
+        throws BundleException
+    {
+        Object[][][] clauses = parseStandardHeader(
+            (String) headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+        if (clauses.length > 0)
+        {
+            if (clauses.length > 1)
+            {
+                throw new BundleException(
+                    "Cannot have multiple symbolic names: "
+                        + headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+            }
+            else if (clauses[0][CLAUSE_PATHS_INDEX].length > 1)
+            {
+                throw new BundleException(
+                    "Cannot have multiple symbolic names: "
+                        + headerMap.get(Constants.BUNDLE_SYMBOLICNAME));
+            }
+
+            // Get bundle version.
+            Version bundleVersion = null;
+            if (headerMap.get(Constants.BUNDLE_VERSION) != null)
+            {
+                try
+                {
+                    bundleVersion = Version.parseVersion((String) headerMap.get(Constants.BUNDLE_VERSION));
+                }
+                catch (RuntimeException ex)
+                {
+                    // R4 bundle versions must parse, R3 bundle version may not.
+                    if (((String) headerMap.get(Constants.BUNDLE_MANIFESTVERSION)).equals("2"))
+                    {
+                        throw ex;
+                    }
+                    bundleVersion = Version.emptyVersion;
+                }
+            }
+
+            // Create a module capability and return it.
+            String symName = (String) clauses[0][CLAUSE_PATHS_INDEX][0];
+            R4Attribute[] attrs = new R4Attribute[2];
+            attrs[0] = new R4Attribute(
+                Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE, symName, false);
+            attrs[1] = new R4Attribute(
+                Constants.BUNDLE_VERSION_ATTRIBUTE, bundleVersion, false);
+            return new Capability(ICapability.MODULE_NAMESPACE, (R4Directive[]) clauses[0][CLAUSE_DIRECTIVES_INDEX], attrs);
+        }
+
+        return null;
+    }
+
     public static ICapability[] parseExportHeader(String header)
     {
         Object[][][] clauses = parseStandardHeader(header);
@@ -826,7 +917,7 @@ public class ManifestParser
                 R4Attribute[] newAttrs = new R4Attribute[attrs.length + 1];
                 newAttrs[0] = new R4Attribute(
                     ICapability.PACKAGE_PROPERTY,
-                    (String) clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
+                    clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
                 System.arraycopy(attrs, 0, newAttrs, 1, attrs.length);
 
                 // Create package capability and add to capability list.
@@ -919,14 +1010,13 @@ public class ManifestParser
                 R4Attribute[] newAttrs = new R4Attribute[attrs.length + 1];
                 newAttrs[0] = new R4Attribute(
                     ICapability.PACKAGE_PROPERTY,
-                    (String) clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
+                    clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
                 System.arraycopy(attrs, 0, newAttrs, 1, attrs.length);
 
                 // Create package requirement and add to requirement list.
                 reqList.add(
                     new Requirement(
                         ICapability.PACKAGE_NAMESPACE,
-                        null,
                         (R4Directive[]) clauses[clauseIdx][CLAUSE_DIRECTIVES_INDEX],
                         newAttrs));
             }
@@ -974,14 +1064,13 @@ public class ManifestParser
                 R4Attribute[] newAttrs = new R4Attribute[attrs.length + 1];
                 newAttrs[0] = new R4Attribute(
                     Constants.BUNDLE_SYMBOLICNAME_ATTRIBUTE,
-                    (String) clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
+                    clauses[clauseIdx][CLAUSE_PATHS_INDEX][pathIdx], false);
                 System.arraycopy(attrs, 0, newAttrs, 1, attrs.length);
 
                 // Create package requirement and add to requirement list.
                 reqList.add(
                     new Requirement(
                         ICapability.MODULE_NAMESPACE,
-                        null,
                         (R4Directive[]) clauses[clauseIdx][CLAUSE_DIRECTIVES_INDEX],
                         newAttrs));
             }
